@@ -11,19 +11,180 @@ import pytest
 from scipy import stats
 
 import semeio
-from semeio.fmudesign import DesignMatrix, excel2dict_design
-from semeio.fmudesign._excel2dict import _read_defaultvalues
-from semeio.fmudesign.create_design import nearest_correlation_matrix
+from semeio.fmudesign import DesignMatrix, excel_to_dict
+from semeio.fmudesign._excel_to_dict import _read_defaultvalues
 
 TESTDATA = Path(__file__).parent / "data"
 
 
+@pytest.mark.parametrize("correlations", [True, False])
+def test_distribution_statistis(tmpdir, monkeypatch, correlations):
+    """This test ensures that if any large-sample statistics for any distribution
+    changes, we will likely pick it up in the future."""
+
+    NUM_SAMPLES = 10**5
+
+    def gl(paramname, distname, p1, p2, p3="", p4=""):
+        """GL = Generate Line. Generates a line in the input sheet."""
+        return [
+            "",
+            "",
+            "",
+            paramname,
+            "",
+            "",
+            "",
+            "",
+            distname,
+            p1,
+            p2,
+            p3,
+            p4,
+            "",
+            "corr1" if correlations else "",
+            "",
+        ]
+
+    # General input sheet
+    general_input = pd.DataFrame(
+        data=[
+            ["designtype", "onebyone"],
+            ["repeats", 1],
+            ["rms_seeds", "default"],
+            ["background", "None"],
+            ["distribution_seed", "None"],
+        ]
+    )
+
+    # Design input sheet
+    design_input = pd.DataFrame(
+        data=[
+            # Normal has params (mean, std, low=-inf, high=inf)
+            gl("NORMAL", "normal", 0, 2),
+            gl("TRUNCNORM", "normal", 0, 1, -1, 2),
+            # Lognormal has params (mean, sigma)
+            gl("LOGNORMAL", "logn", 1.5, 0.5),
+            # Uniform has params (low, high)
+            gl("UNIFORM", "unif", -5, 0),
+            # Triangular has params (low, mode, high)
+            gl("TRIANG", "triang", -5, 0, 5),
+            # Pert has has params (low, mode, high, scale=4)
+            gl("DEFAULTPERT", "pert", -5, 0, 5),
+            gl("SCALEPERT", "pert", -5, 0, 5, 1),
+            # Loguniform has params (low, high)
+            gl("LOGUNIFORM", "logunif", 1, 5),
+        ],
+        columns=[
+            "sensname",
+            "numreal",
+            "type",
+            "param_name",
+            "senscase1",
+            "value1",
+            "senscase2",
+            "value2",
+            "dist_name",
+            "dist_param1",
+            "dist_param2",
+            "dist_param3",
+            "dist_param4",
+            "decimals",
+            "corr_sheet",
+            "extern_file",
+        ],
+    )
+    design_input.iloc[0, :3] = ["distr_test", (NUM_SAMPLES), "dist"]
+
+    # Default values sheet
+    defaultvalues = pd.DataFrame(
+        {
+            "param_name": list(design_input["param_name"]),
+            "default_value": [0.5] * (len(design_input)),
+        }
+    )
+
+    # Correlation sheet
+    num_vars = len(design_input["param_name"])
+    corr_values = np.zeros(shape=(num_vars, num_vars)) + 0.2
+    np.fill_diagonal(corr_values, val=1.0)
+    upper_idx = np.triu_indices_from(corr_values, k=1)
+    corr_sheet = pd.DataFrame(
+        corr_values,
+        columns=list(design_input["param_name"]),
+        index=list(design_input["param_name"]),
+    ).astype(str)
+    corr_sheet.values[upper_idx] = ""  # Set upper triang to blank
+
+    # Create a file to do the save => load roundtrip and test that too
+    FILENAME = "designinput.xlsx"
+    with pd.ExcelWriter(FILENAME, engine="openpyxl") as writer:
+        general_input.to_excel(
+            writer, sheet_name="general_input", index=False, header=None
+        )
+        design_input.to_excel(writer, sheet_name="designinput", index=False)
+        defaultvalues.to_excel(writer, sheet_name="defaultvalues", index=False)
+        corr_sheet.to_excel(writer, sheet_name="corr1")
+
+    # Read the file and draw samples
+    input_dict = excel_to_dict(FILENAME)
+    design = DesignMatrix()
+    design.generate(input_dict)
+    assert len(design.designvalues) == NUM_SAMPLES
+    df = design.designvalues
+
+    # Test statistical properties and boundaries of all variables.
+    # There were either derived using analytical properties, or empirically
+    # by drawing 10 million samples.
+    # Tolerance must be high enough to not pick up on rng differences, but low
+    # enough to pick up meaningful changes.
+    atol = 0.005
+
+    assert np.isclose(df["NORMAL"].mean(), 0.0, atol=atol)
+    assert np.isclose(df["NORMAL"].std(), 2.0, atol=atol)
+
+    assert np.isclose(df["TRUNCNORM"].mean(), 0.229637, atol=atol)
+    assert np.isclose(df["TRUNCNORM"].std(), 0.720945, atol=atol)
+    assert df["TRUNCNORM"].min() >= -1
+    assert df["TRUNCNORM"].max() <= 2
+
+    assert np.isclose(df["LOGNORMAL"].mean(), 5.078418, atol=atol)
+    assert np.isclose(df["LOGNORMAL"].std(), 2.706487, atol=atol)
+
+    assert df["UNIFORM"].min() >= -5
+    assert df["UNIFORM"].max() <= 0
+    assert np.isclose(df["UNIFORM"].mean(), -2.5, atol=atol)
+    assert np.isclose(df["UNIFORM"].std(), 1.443375, atol=atol)
+
+    assert df["TRIANG"].min() >= -5
+    assert df["TRIANG"].max() <= 5
+    assert np.isclose(df["TRIANG"].mean(), 0, atol=atol)
+    assert np.isclose(df["TRIANG"].std(), 2.041241, atol=atol)
+
+    assert df["DEFAULTPERT"].min() >= -5
+    assert df["DEFAULTPERT"].max() <= 5
+    assert np.isclose(df["DEFAULTPERT"].mean(), 0, atol=atol)
+    assert np.isclose(df["DEFAULTPERT"].std(), 1.889822, atol=atol)
+
+    assert df["SCALEPERT"].min() >= -5
+    assert df["SCALEPERT"].max() <= 5
+    assert np.isclose(df["SCALEPERT"].mean(), 0, atol=atol)
+    assert np.isclose(df["SCALEPERT"].std(), 2.5, atol=atol)
+
+    assert np.isclose(df["LOGUNIFORM"].mean(), 2.485339, atol=atol)
+    assert np.isclose(df["LOGUNIFORM"].std(), 1.130975, atol=atol)
+
+    # Check that correlations are close
+    if correlations:
+        obs_corr = df[design_input["param_name"]].corr().to_numpy()
+        assert np.sqrt(np.mean((obs_corr - corr_values) ** 2)) < 0.02
+
+
 def test_generate_onebyone(tmpdir):
     """Test generation of onebyone design"""
 
     inputfile = TESTDATA / "config/design_input_example1.xlsx"
 
-    input_dict = excel2dict_design(inputfile)
+    input_dict = excel_to_dict(inputfile)
 
     # Note that repeats are set to 10 in general_input sheet.
     # So, there are 10 rows for each senscase of type seed and scenario.
@@ -66,7 +227,7 @@ def test_generate_onebyone(tmpdir):
         + ["multz"] * 20
     )
     assert (diskdesign["SENSNAME"] == sensname).all()
-    # Sensitivites of type seed like rms_seed automatically get senscase p10_p90,
+    # Sensitivities of type seed like rms_seed automatically get senscase p10_p90,
     # so that P10/P90 is calculated for the tornado plot.
     assert (
         diskdesign[diskdesign["SENSNAME"] == "rms_seed"]["SENSCASE"] == "p10_p90"
@@ -196,15 +357,20 @@ def test_generate_full_mc_snapshot(snapshot):
     """
     # Setup
     inputfile = TESTDATA / "config/design_input_mc_with_correls.xlsx"
-    input_dict = excel2dict_design(inputfile)
+    input_dict = excel_to_dict(inputfile)
     design = DesignMatrix()
 
     # Generate the design matrix
     design.generate(input_dict)
 
+    # Round to N significant figures
+    df_rounded = design.designvalues.map(
+        lambda x: x if isinstance(x, str) else float(f"{x:.6g}")
+    )
+
     # Prepare data for snapshot comparison
     snapshot_dict = {
-        "designvalues": design.designvalues.to_dict("records"),
+        "designvalues": df_rounded.to_dict("records"),
         "defaultvalues": dict(design.defaultvalues),
     }
 
@@ -222,7 +388,7 @@ def test_generate_full_mc_snapshot(snapshot):
 def test_generate_full_mc(tmpdir):
     """Test generation of full monte carlo"""
     inputfile = TESTDATA / "config/design_input_mc_with_correls.xlsx"
-    input_dict = excel2dict_design(inputfile)
+    input_dict = excel_to_dict(inputfile)
 
     design = DesignMatrix()
     design.generate(input_dict)
@@ -294,7 +460,7 @@ def test_generate_full_mc(tmpdir):
 
 def test_generate_background(tmpdir):
     inputfile = TESTDATA / "config/design_input_background.xlsx"
-    input_dict = excel2dict_design(inputfile)
+    input_dict = excel_to_dict(inputfile)
     source_file = TESTDATA / "config/doe1.xlsx"
     dest_file = tmpdir.join("doe1.xlsx")
     shutil.copy2(source_file, dest_file)
@@ -303,7 +469,7 @@ def test_generate_background(tmpdir):
         design = DesignMatrix()
         design.generate(input_dict)
 
-        # Check that background parameters have same values in different sensitivies.
+        # Check that background parameters have same values in different sensitivities.
         background_params = ["PARAM17", "PARAM18", "PARAM19"]
 
         background_vals = design.designvalues.loc[
@@ -353,81 +519,6 @@ def test_generate_background(tmpdir):
         )
 
 
-class TestNearestCorrelationMatrix:
-    @pytest.mark.parametrize("variables", range(2, 100, 10))
-    def test_nearest_correlation_matrix(self, variables):
-        """Test that we can cholesky decompose the solution."""
-
-        rng = np.random.default_rng(variables)
-
-        # Create a correlation matrix
-        observations = rng.normal(size=(variables * 2, variables))
-        matrix = np.corrcoef(observations, rowvar=False)
-
-        # Taking the cholesky decomposition should work just fine
-        np.linalg.cholesky(matrix)
-
-        # Mess it up
-        matrix = matrix + rng.normal(size=matrix.shape, scale=0.1)
-        matrix = matrix - np.identity(variables) * np.mean(np.diag(matrix))
-
-        # Now the cholesky decomposition should fail
-        with pytest.raises(np.linalg.LinAlgError):
-            np.linalg.cholesky(matrix)
-
-        # Adjust the matrix to its nearest correlation matrix
-        correlation_matrix = nearest_correlation_matrix(matrix)
-
-        # Taking the cholesky decomposition should work now
-        np.linalg.cholesky(correlation_matrix)
-
-        # Diagonal entries should be 1.0 and the matrix should be symmetric
-        assert np.allclose(np.diag(correlation_matrix), 1.0)
-        assert np.allclose(correlation_matrix, correlation_matrix.T)
-
-    def test_nearest_correlation_matrix_on_matlab_example(self):
-        """These matrices are from the 'nearcorr' docs:
-        https://www.mathworks.com/help/stats/nearcorr.html
-        """
-        # The matrix we want to adjust to become a correlation matrix
-        A = np.array(
-            [
-                [1.0, 0.0, 0.0, 0.0, -0.936],
-                [0.0, 1.0, -0.55, -0.3645, -0.53],
-                [0.0, -0.55, 1.0, -0.0351, 0.0875],
-                [0.0, -0.3645, -0.0351, 1.0, 0.4557],
-                [-0.936, -0.53, 0.0875, 0.4557, 1.0],
-            ]
-        )
-
-        W = np.array(
-            [
-                [0.0, 1.0, 0.1, 0.15, 0.25],
-                [1.0, 0.0, 0.05, 0.025, 0.15],
-                [0.1, 0.05, 0.0, 0.25, 1.0],
-                [0.15, 0.025, 0.25, 0.0, 0.25],
-                [0.25, 0.15, 1.0, 0.25, 0.0],
-            ]
-        )
-
-        matlab_Y = np.array(
-            [
-                [1.0, 0.0014, 0.0287, -0.0222, -0.8777],
-                [0.0014, 1.0, -0.498, -0.7268, -0.4567],
-                [0.0287, -0.498, 1.0, -0.0358, 0.0878],
-                [-0.0222, -0.7268, -0.0358, 1.0, 0.4465],
-                [-0.8777, -0.4567, 0.0878, 0.4465, 1.0],
-            ]
-        )
-
-        # The smallest eigenvalue of A is -0.1244...
-        # The smallest eigenvalue of Y is 1.088e-06
-        Y = nearest_correlation_matrix(A, weights=W)
-
-        # Matlab output has 4 digits, so atol is set to 1e-4 here
-        assert np.allclose(Y, matlab_Y, atol=1e-4)
-
-
 def test_read_defaultvalues_duplicate_error(tmpdir, monkeypatch):
     """Test that read_defaultvalues raises ValueError for duplicate parameter names."""
     monkeypatch.chdir(tmpdir)
