@@ -8,7 +8,8 @@ from pathlib import Path
 import pandas as pd
 import pytest
 
-from semeio.fmudesign import DesignMatrix, excel2dict_design
+from semeio.fmudesign import DesignMatrix, excel_to_dict
+from semeio.fmudesign.fmudesignrunner import EXAMPLE_FILES
 
 TESTDATA = Path(__file__).parent / "data"
 TEST_FILES = list((TESTDATA / "config").glob("design_input*.xlsx"))
@@ -77,7 +78,7 @@ def test_prediction_rejection_sampled_ensemble(tmpdir, monkeypatch):
     defaultvalues.to_excel(writer, sheet_name="defaultvalues", index=False)
     writer.close()
 
-    dict_design = excel2dict_design("designinput.xlsx")
+    dict_design = excel_to_dict("designinput.xlsx")
     design = DesignMatrix()
     design.generate(dict_design)
 
@@ -149,7 +150,7 @@ def test_constant_distribution(tmpdir, monkeypatch, gen_input_sheet):
     writer.close()
 
     # Generate design matrix
-    dict_design = excel2dict_design("designinput.xlsx", gen_input_sheet="generalinput")
+    dict_design = excel_to_dict("designinput.xlsx", gen_input_sheet="generalinput")
     design = DesignMatrix()
     design.generate(dict_design)
 
@@ -160,7 +161,7 @@ def test_constant_distribution(tmpdir, monkeypatch, gen_input_sheet):
 
 
 @pytest.mark.parametrize("designfile", TEST_FILES, ids=[p.stem for p in TEST_FILES])
-@pytest.mark.parametrize("verbosity", [0, 1])
+@pytest.mark.parametrize("verbosity", [0, 1, 2])
 def test_all_input_files(tmpdir, monkeypatch, designfile, verbosity):
     """Smoketest all files."""
 
@@ -178,6 +179,25 @@ def test_all_input_files(tmpdir, monkeypatch, designfile, verbosity):
     )
 
 
+@pytest.mark.parametrize("designfile", EXAMPLE_FILES.keys(), ids=EXAMPLE_FILES.keys())
+@pytest.mark.parametrize("verbosity", [0, 1])
+def test_all_example_files_cmd_init(tmpdir, monkeypatch, designfile, verbosity):
+    """Smoketest all files available in fmudesign init subcommand."""
+    monkeypatch.chdir(tmpdir)
+    subprocess.run(
+        ["fmudesign", "init", designfile], check=True, capture_output=True, text=True
+    )
+
+    # Run the CLI tool (test will fail on non-zero status code)
+    verbose = ["--verbose"] * verbosity
+    subprocess.run(
+        ["fmudesign", "run", designfile, *verbose],
+        check=True,
+        capture_output=True,
+        text=True,
+    )
+
+
 @pytest.mark.parametrize("designfile", TEST_FILES, ids=[p.stem for p in TEST_FILES])
 def test_all_input_files_relative_paths(tmpdir, monkeypatch, designfile):
     """Smoketest all files, but invoke them from a directory above.
